# encoding: utf-8
#
# Copyright 2016 Matt Wrock <matt@mattwrock.com>
# Copyright 2016 Sam Oluwalana <soluwalana@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require_relative 'base'

module PSRP
  module WSMV


    # WSMV message to create a remote shell
    class InitRunspacePool < Base
      attr_accessor :shell_id

      def initialize(session_opts)
        @session_opts = session_opts
        @shell_id = SecureRandom.uuid.to_s.upcase
      end

      protected

      def create_header(header)
        header << Gyoku.xml(shell_headers)
      end

      def create_body(body)
        body.tag!("#{NS_WIN_SHELL}:Shell") { |s| s << Gyoku.xml(shell_body) }
      end

      private

      def shell_body
        body = {
          "#{NS_WIN_SHELL}:InputStreams" => 'stdin pr',
          "#{NS_WIN_SHELL}:OutputStreams" => 'stdout'
        }
        session_capabilities = PSRP::MessageEncoder.new(shell_id, nil, :SESSION_CAPABILITY)
        runspace_init = PSRP::MessageEncoder.new(shell_id, nil, :INIT_RUNSPACEPOOL)
        body['creationXml'] = encode_bytes(session_capabilities.fragments[0] + runspace_init.fragments[0])
        body[:attributes!] = {
          'creationXml' => {
            'xmlns' => 'http://schemas.microsoft.com/powershell'
          }
        }
        body
      end

      def header_opts
        {
          "#{NS_WSMAN_DMTF}:OptionSet" => {
            "#{NS_WSMAN_DMTF}:Option" => 2.1,
            :attributes! => {
              "#{NS_WSMAN_DMTF}:Option" => {
                'Name' => 'protocolversion',
                'MustComply' => 'true'
              }
            }
          },
          :attributes! => {
            "#{NS_WSMAN_DMTF}:OptionSet" => {
              'env:mustUnderstand' => 'true'
            }
          }
        }
      end

      def shell_headers
        merge_headers(shared_headers(@session_opts),
                      resource_uri_shell(RESOURCE_URI_POWERSHELL),
                      header_opts,
                      action_create,
                      selector_shell_id(shell_id))

      end

   
    end
  end
end
